declare type ArgumentsType<T> = T extends (...args: infer A) => any ? A : never;
declare type ReturnType<T> = T extends (...args: any) => infer R ? R : never;
interface ChannelOptions {
    /**
     * Function to post raw message
     */
    post: (data: any, ...extras: any[]) => void;
    /**
     * Listener to receive raw message
     */
    on: (fn: (data: any, ...extras: any[]) => void) => void;
    /**
     * Custom function to serialize data
     *
     * by default it passes the data as-is
     */
    serialize?: (data: any) => any;
    /**
     * Custom function to deserialize data
     *
     * by default it passes the data as-is
     */
    deserialize?: (data: any) => any;
}
interface EventOptions<Remote> {
    /**
     * Names of remote functions that do not need response.
     */
    eventNames?: (keyof Remote)[];
    /**
     * Maximum timeout for waiting for response, in milliseconds.
     *
     * @default 60_000
     */
    timeout?: number;
}
declare type BirpcOptions<Remote> = EventOptions<Remote> & ChannelOptions;
interface BirpcFn<T> {
    /**
     * Call the remote function and wait for the result.
     */
    (...args: ArgumentsType<T>): Promise<Awaited<ReturnType<T>>>;
    /**
     * Send event without asking for response
     */
    asEvent(...args: ArgumentsType<T>): void;
}
interface BirpcGroupFn<T> {
    /**
     * Call the remote function and wait for the result.
     */
    (...args: ArgumentsType<T>): Promise<Awaited<ReturnType<T>>[]>;
    /**
     * Send event without asking for response
     */
    asEvent(...args: ArgumentsType<T>): void;
}
declare type BirpcReturn<RemoteFunctions> = {
    [K in keyof RemoteFunctions]: BirpcFn<RemoteFunctions[K]>;
};
declare type BirpcGroupReturn<RemoteFunctions> = {
    [K in keyof RemoteFunctions]: BirpcGroupFn<RemoteFunctions[K]>;
};
declare const DEFAULT_TIMEOUT = 60000;
declare function createBirpc<RemoteFunctions = {}, LocalFunctions = {}>(functions: LocalFunctions, options: BirpcOptions<RemoteFunctions>): BirpcReturn<RemoteFunctions>;
declare function cachedMap<T, R>(items: T[], fn: ((i: T) => R)): R[];
declare function createBirpcGroup<RemoteFunctions = {}, LocalFunctions = {}>(functions: LocalFunctions, channels: ChannelOptions[] | (() => ChannelOptions[]), options?: EventOptions<RemoteFunctions>): {
    readonly clients: BirpcReturn<RemoteFunctions>[];
    updateChannels: (fn?: ((channels: ChannelOptions[]) => void) | undefined) => BirpcReturn<RemoteFunctions>[];
    boardcast: BirpcGroupReturn<RemoteFunctions>;
};

export { ArgumentsType, BirpcFn, BirpcGroupFn, BirpcGroupReturn, BirpcOptions, BirpcReturn, ChannelOptions, DEFAULT_TIMEOUT, EventOptions, ReturnType, cachedMap, createBirpc, createBirpcGroup };
